@@ -1,26 +1,19 @@
-{-# LANGUAGE BangPatterns, CPP, ForeignFunctionInterface, MagicHash,
-             ScopedTypeVariables, UnliftedFFITypes #-}
-
-#if __GLASGOW_HASKELL__ < 710
-{-# LANGUAGE DeriveDataTypeable #-}
-#endif
+{-# LANGUAGE BangPatterns, CPP, MagicHash,
+             ScopedTypeVariables, UnliftedFFITypes, DeriveDataTypeable,
+             DefaultSignatures, FlexibleContexts, TypeFamilies,
+             MultiParamTypeClasses #-}
 
 #if __GLASGOW_HASKELL__ >= 801
 {-# LANGUAGE PolyKinds #-} -- For TypeRep instances
 #endif
 
-#ifdef GENERICS
-{-# LANGUAGE DefaultSignatures, FlexibleContexts, GADTs,
-    MultiParamTypeClasses, EmptyDataDecls #-}
-#endif
-
 ------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Hashable.Class
 -- Copyright   :  (c) Milan Straka 2010
 --                (c) Johan Tibell 2011
 --                (c) Bryan O'Sullivan 2011, 2012
--- License     :  BSD-style
+-- SPDX-License-Identifier : BSD-3-Clause
 -- Maintainer  :  johan.tibell@gmail.com
 -- Stability   :  provisional
 -- Portability :  portable
@@ -36,13 +29,14 @@ module Data.Hashable.Class
       Hashable(..)
     , Hashable1(..)
     , Hashable2(..)
-#ifdef GENERICS
+
       -- ** Support for generics
+    , genericHashWithSalt
+    , genericLiftHashWithSalt
     , GHashable(..)
     , HashArgs(..)
     , Zero
     , One
-#endif
 
       -- * Creating new instances
     , hashUsing
@@ -106,25 +100,21 @@ import Data.Fixed (Fixed(..))
 import Data.Functor.Identity (Identity(..))
 #endif
 
-#ifdef GENERICS
 import GHC.Generics
-#endif
 
-#if   __GLASGOW_HASKELL__ >= 801
+#if   MIN_VERSION_base(4,10,0)
 import Type.Reflection (Typeable, TypeRep, SomeTypeRep(..))
 import Type.Reflection.Unsafe (typeRepFingerprint)
 import GHC.Fingerprint.Type(Fingerprint(..))
-#elif __GLASGOW_HASKELL__ >= 710
+#elif MIN_VERSION_base(4,8,0)
 import Data.Typeable (typeRepFingerprint, Typeable, TypeRep)
 import GHC.Fingerprint.Type(Fingerprint(..))
-#elif __GLASGOW_HASKELL__ >= 702
+#else
 import Data.Typeable.Internal (Typeable, TypeRep (..))
 import GHC.Fingerprint.Type(Fingerprint(..))
-#elif __GLASGOW_HASKELL__ >= 606
-import Data.Typeable (typeRepKey, Typeable, TypeRep)
 #endif
 
-#if __GLASGOW_HASKELL__ >= 703
+#if MIN_VERSION_base(4,5,0)
 import Foreign.C (CLong(..))
 import Foreign.C.Types (CInt(..))
 #else
@@ -233,30 +223,42 @@ class Hashable a where
     hash :: a -> Int
     hash = hashWithSalt defaultSalt
 
-#ifdef GENERICS
     default hashWithSalt :: (Generic a, GHashable Zero (Rep a)) => Int -> a -> Int
-    hashWithSalt salt = ghashWithSalt HashArgs0 salt . from
+    hashWithSalt = genericHashWithSalt
+    {-# INLINE hashWithSalt #-}
+
+-- | Generic 'hashWithSalt'.
+--
+-- @since 1.3.0.0
+genericHashWithSalt :: (Generic a, GHashable Zero (Rep a)) => Int -> a -> Int
+genericHashWithSalt = \salt -> ghashWithSalt HashArgs0 salt . from
+{-# INLINE genericHashWithSalt #-}
 
 data Zero
 data One
 
-data HashArgs arity a where
-    HashArgs0 :: HashArgs Zero a
-    HashArgs1 :: (Int -> a -> Int) -> HashArgs One a
+data family HashArgs arity a :: *
+data instance HashArgs Zero a = HashArgs0
+newtype instance HashArgs One  a = HashArgs1 (Int -> a -> Int)
 
 -- | The class of types that can be generically hashed.
 class GHashable arity f where
     ghashWithSalt :: HashArgs arity a -> Int -> f a -> Int
 
-#endif
-
 class Hashable1 t where
     -- | Lift a hashing function through the type constructor.
     liftHashWithSalt :: (Int -> a -> Int) -> Int -> t a -> Int
-#ifdef GENERICS
+
     default liftHashWithSalt :: (Generic1 t, GHashable One (Rep1 t)) => (Int -> a -> Int) -> Int -> t a -> Int
-    liftHashWithSalt h salt = ghashWithSalt (HashArgs1 h) salt . from1
-#endif
+    liftHashWithSalt = genericLiftHashWithSalt
+    {-# INLINE liftHashWithSalt #-}
+
+-- | Generic 'liftHashWithSalt'.
+--
+-- @since 1.3.0.0
+genericLiftHashWithSalt :: (Generic1 t, GHashable One (Rep1 t)) => (Int -> a -> Int) -> Int -> t a -> Int
+genericLiftHashWithSalt = \h salt -> ghashWithSalt (HashArgs1 h) salt . from1
+{-# INLINE genericLiftHashWithSalt #-}
 
 class Hashable2 t where
     -- | Lift a hashing function through the binary type constructor.
@@ -458,8 +460,14 @@ instance (Integral a, Hashable a) => Hashable (Ratio a) where
     hash a = hash (numerator a) `hashWithSalt` denominator a
     hashWithSalt s a = s `hashWithSalt` numerator a `hashWithSalt` denominator a
 
+-- | __Note__: prior to @hashable-1.3.0.0@, @hash 0.0 /= hash (-0.0)@
+--
+-- The 'hash' of NaN is not well defined.
+--
+-- @since 1.3.0.0
 instance Hashable Float where
     hash x
+        | x == -0.0 || x == 0.0 = 0 -- see note in 'Hashable Double'
         | isIEEE x =
             assert (sizeOf x >= sizeOf (0::Word32) &&
                     alignment x >= alignment (0::Word32)) $
@@ -467,8 +475,14 @@ instance Hashable Float where
         | otherwise = hash (show x)
     hashWithSalt = defaultHashWithSalt
 
+-- | __Note__: prior to @hashable-1.3.0.0@, @hash 0.0 /= hash (-0.0)@
+--
+-- The 'hash' of NaN is not well defined.
+--
+-- @since 1.3.0.0
 instance Hashable Double where
     hash x
+        | x == -0.0 || x == 0.0 = 0 -- s.t. @hash -0.0 == hash 0.0@ ; see #173
         | isIEEE x =
             assert (sizeOf x >= sizeOf (0::Word64) &&
                     alignment x >= alignment (0::Word64)) $
@@ -620,11 +634,7 @@ instance Hashable BL.ByteString where
 
 #if MIN_VERSION_bytestring(0,10,4)
 instance Hashable BSI.ShortByteString where
-#if MIN_VERSION_base(4,3,0)
     hashWithSalt salt sbs@(BSI.SBS ba) =
-#else
-    hashWithSalt salt sbs@(BSI.SBS ba _) =
-#endif
         hashByteArrayWithSalt ba 0 (BSI.length sbs) salt
 #endif
 
@@ -661,28 +671,16 @@ instance Hashable WordPtr where
     hash n = fromIntegral n
     hashWithSalt = defaultHashWithSalt
 
-#if __GLASGOW_HASKELL__ < 801
--- | Compute the hash of a TypeRep, in various GHC versions we can do this quickly.
-hashTypeRep :: TypeRep -> Int
-{-# INLINE hashTypeRep #-}
-#if __GLASGOW_HASKELL__ >= 710
--- Fingerprint is just the MD5, so taking any Int from it is fine
-hashTypeRep tr = let Fingerprint x _ = typeRepFingerprint tr in fromIntegral x
-#elif __GLASGOW_HASKELL__ >= 702
--- Fingerprint is just the MD5, so taking any Int from it is fine
-hashTypeRep (TypeRep (Fingerprint x _) _ _) = fromIntegral x
-#elif __GLASGOW_HASKELL__ >= 606
-hashTypeRep = unsafeDupablePerformIO . typeRepKey
-#else
-hashTypeRep = hash . show
-#endif
+----------------------------------------------------------------------------
+-- Fingerprint & TypeRep instances
 
-instance Hashable TypeRep where
-    hash = hashTypeRep
+-- | @since 1.3.0.0
+instance Hashable Fingerprint where
+    hash (Fingerprint x _) = fromIntegral x
     hashWithSalt = defaultHashWithSalt
     {-# INLINE hash #-}
 
-#else
+#if MIN_VERSION_base(4,10,0)
 
 hashTypeRep :: Type.Reflection.TypeRep a -> Int
 hashTypeRep tr =
@@ -697,8 +695,29 @@ instance Hashable (Type.Reflection.TypeRep a) where
     hash = hashTypeRep
     hashWithSalt = defaultHashWithSalt
     {-# INLINE hash #-}
+
+#else
+
+-- | Compute the hash of a TypeRep, in various GHC versions we can do this quickly.
+hashTypeRep :: TypeRep -> Int
+{-# INLINE hashTypeRep #-}
+#if   MIN_VERSION_base(4,8,0)
+-- Fingerprint is just the MD5, so taking any Int from it is fine
+hashTypeRep tr = let Fingerprint x _ = typeRepFingerprint tr in fromIntegral x
+#else
+-- Fingerprint is just the MD5, so taking any Int from it is fine
+hashTypeRep (TypeRep (Fingerprint x _) _ _) = fromIntegral x
 #endif
 
+instance Hashable TypeRep where
+    hash = hashTypeRep
+    hashWithSalt = defaultHashWithSalt
+    {-# INLINE hash #-}
+
+#endif
+
+----------------------------------------------------------------------------
+
 #if MIN_VERSION_base(4,8,0)
 instance Hashable Void where
     hashWithSalt _ = absurd
@@ -813,8 +832,11 @@ instance Hashable a => Hashable (Min a) where
 instance Hashable a => Hashable (Max a) where
     hashWithSalt p (Max a) = hashWithSalt p a
 
-instance (Hashable a, Hashable b) => Hashable (Arg a b) where
-    hashWithSalt p (Arg a b) = hashWithSalt p a `hashWithSalt` b
+-- | __Note__: Prior to @hashable-1.3.0.0@ the hash computation included the second argument of 'Arg' which wasn't consistent with its 'Eq' instance.
+--
+-- @since 1.3.0.0
+instance Hashable a => Hashable (Arg a b) where
+    hashWithSalt p (Arg a _) = hashWithSalt p a
 
 instance Hashable a => Hashable (First a) where
     hashWithSalt p (First a) = hashWithSalt p a
@@ -916,5 +938,3 @@ instance Ord1 Hashed where
 instance Show1 Hashed where
   liftShowsPrec sp _ d (Hashed a _) = showsUnaryWith sp "hashed" d a
 #endif
-
-
