@@ -1,7 +1,4 @@
-{-# LANGUAGE CPP #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
-#endif
 
 ------------------------------------------------------------------------
 -- |
@@ -9,7 +6,7 @@
 -- Copyright   :  (c) Milan Straka 2010
 --                (c) Johan Tibell 2011
 --                (c) Bryan O'Sullivan 2011, 2012
--- License     :  BSD-style
+-- SPDX-License-Identifier : BSD-3-Clause
 -- Maintainer  :  johan.tibell@gmail.com
 -- Stability   :  provisional
 -- Portability :  portable
@@ -77,7 +74,7 @@ import Data.Hashable.Class
 -- given a 'liftHash' method:
 --
 -- > class Hashable1 t where
--- >     liftHash :: (Int -> a) -> t a -> Int
+-- >     liftHash :: (a -> Int) -> t a -> Int
 -- >     liftHashWithSalt :: (Int -> a -> Int) -> Int -> t a -> Int
 --
 -- Even for a type as simple as 'Maybe', the problem manifests itself. The
