@@ -1,15 +1,16 @@
 {-# LANGUAGE BangPatterns, CPP, GeneralizedNewtypeDeriving, MagicHash,
     Rank2Types, UnboxedTuples #-}
-#ifdef GENERICS
 {-# LANGUAGE DeriveGeneric, ScopedTypeVariables #-}
-#endif
 
 -- | QuickCheck tests for the 'Data.Hashable' module.  We test
 -- functions by comparing the C and Haskell implementations.
 
 module Properties (properties) where
 
-import Data.Hashable (Hashable, hash, hashByteArray, hashPtr)
+import Data.Hashable (Hashable, hash, hashByteArray, hashPtr,
+         Hashed, hashed, unhashed, hashWithSalt)
+import Data.Hashable.Generic (genericHashWithSalt)
+import Data.Hashable.Lifted (hashWithSalt1)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.Text as T
@@ -25,9 +26,7 @@ import GHC.Word (Word8(..))
 import Test.QuickCheck hiding ((.&.))
 import Test.Framework (Test, testGroup)
 import Test.Framework.Providers.QuickCheck2 (testProperty)
-#ifdef GENERICS
 import GHC.Generics
-#endif
 
 #if MIN_VERSION_bytestring(0,10,4)
 import qualified Data.ByteString.Short as BS
@@ -154,8 +153,6 @@ fromList xs0 = unBA (runST $ ST $ \ s1# ->
 
 -- Generics
 
-#ifdef GENERICS
-
 data Product2 a b = Product2 a b
                     deriving (Generic)
 
@@ -189,6 +186,13 @@ instance (Hashable a, Hashable b) => Hashable (Sum2 a b)
 data Sum3 a b c = S3a a | S3b b | S3c c
                   deriving (Eq, Ord, Show, Generic)
 
+instance (Arbitrary a, Arbitrary b, Arbitrary c) => Arbitrary (Sum3 a b c) where
+  arbitrary = oneof
+    [ fmap S3a arbitrary
+    , fmap S3b arbitrary
+    , fmap S3c arbitrary
+    ]
+
 instance (Hashable a, Hashable b, Hashable c) => Hashable (Sum3 a b c)
 
 -- Hashes of the same parameter, but with different sum constructors,
@@ -206,7 +210,15 @@ pSum3_differ x = nub hs == hs
              , hash (S3b x :: Sum3 Int Int Int)
              , hash (S3c x :: Sum3 Int Int Int) ]
 
-#endif
+pGeneric :: Sum3 Int Bool String -> Int -> Bool
+pGeneric x salt = hashWithSalt salt x == genericHashWithSalt salt x
+
+instance (Arbitrary a, Hashable a) => Arbitrary (Hashed a) where
+  arbitrary = fmap hashed arbitrary
+  shrink xs = map hashed $ shrink $ unhashed xs
+
+pLiftedHashed :: Int -> Hashed (Either Int String) -> Bool
+pLiftedHashed s h = hashWithSalt s h == hashWithSalt1 s h
 
 properties :: [Test]
 properties =
@@ -226,7 +238,6 @@ properties =
       , testProperty "bytestring/rechunk" pBSRechunk
       , testProperty "bytestring/rechunked" pBSLazyRechunked
       ]
-#ifdef GENERICS
     , testGroup "generics"
       [
       -- Note: "product2" and "product3" have been temporarily
@@ -237,8 +248,11 @@ properties =
       -- "product3" pProduct3
         testProperty "sum2_differ" pSum2_differ
       , testProperty "sum3_differ" pSum3_differ
+      , testProperty "genericHashWithSalt" pGeneric
+      ]
+    , testGroup "lifted law"
+      [ testProperty "Hashed" pLiftedHashed
       ]
-#endif
     ]
 
 ------------------------------------------------------------------------
